@@ -3,8 +3,8 @@ import typing
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from enum import Enum
-import pandas as pd
 
+import pandas as pd
 from sqlalchemy import Table, Column
 from sqlalchemy.sql import FromClause
 from sqlalchemy.sql.elements import BooleanClauseList
@@ -22,9 +22,8 @@ date_constraint_t = typing.Union[
     datetime.datetime,
 ]
 
-
 DT_COLUMN_NAME = 'dt'
-DEFAULT_SERIES = ['B', 'D', 'W', 'M', 'SM', 'BM', 'MS', 'SMS', 'BMS', 'Q', 'BQ', 'QS',  'Y', 'BY', 'YS', 'BYS']
+DEFAULT_SERIES = ['B', 'D', 'W', 'M', 'SM', 'BM', 'MS', 'SMS', 'BMS', 'Q', 'BQ', 'QS', 'Y', 'BY', 'YS', 'BYS']
 INFINITY_PROXY_DATE = datetime.datetime(year=3000, month=12, day=31)
 
 
@@ -116,7 +115,8 @@ class DataStoreHelper(ABC):
         pass
 
     @abstractmethod
-    def get_series_table(self, frequency: typing.Tuple[datetime.datetime, datetime.datetime, str]) -> Table:
+    def get_series_table(self, frequency: typing.Tuple[datetime.datetime, datetime.datetime, str],
+                         alias: str = None) -> Table:
         pass
 
 
