@@ -22,7 +22,7 @@ from sqlalchemy import (
 from sqlalchemy.dialects import postgresql
 from sqlalchemy.dialects.postgresql.base import BYTEA
 from sqlalchemy.schema import CreateTable
-from sqlalchemy.sql.ddl import CreateIndex
+from sqlalchemy.sql.ddl import CreateIndex, DropTable
 from sqlalchemy.sql.sqltypes import (
     BOOLEAN,
     DATE,
@@ -35,11 +35,23 @@ from sqlalchemy.sql.sqltypes import (
 )
 from sqlalchemy_redshift.dialect import RedshiftDialect
 
-UP_INITIAL_MIGRATION_NAME = "1_initial.up.sql"
-TABLES_MIGRATION_NAME = "2_tables.up.sql"
+UP_TABLES_MIGRATION_NAME = "1_tables.up.sql"
+DOWN_TABLES_MIGRATION_NAME = "1_tables.down.sql"
 
 
 class MigrationProvider(ABC):
+
+    def create_table(self, name: str, schema: str, columns: typing.List[Column], primary_key_columns: typing.List[str],
+                     **kwargs) -> Table:
+        return Table(
+            name,
+            self.get_metadata(),
+            *columns,
+            PrimaryKeyConstraint(*primary_key_columns),
+            schema=schema,
+            **kwargs,
+        )
+
     def get_dating_columns(self):
         return [
             "effective_start_dt",
@@ -83,7 +95,12 @@ class MigrationProvider(ABC):
         ]
 
     def create_partition_table(
-        self, f, schema: str, parent_name: str, _from: str, to: str
+            self, f, schema: str, parent_name: str, _from: str, to: str
+    ):
+        pass
+
+    def drop_partition_table(
+            self, f, schema: str, parent_name: str, _from: str, to: str
     ):
         pass
 
@@ -101,19 +118,41 @@ class PostgresProvider(MigrationProvider):
     def get_name(self):
         return "postgres"
 
-    def create_partition_table(
-        self, f, schema: str, parent_name: str, _from: str, to: str
+    def drop_partition_table(
+            self, f, schema: str, parent_name: str, _from: str, to: str
     ):
-        stmt = f"CREATE TABLE IF NOT EXISTS {schema}.{parent_name}_p{_from}_{to} PARTITION OF {schema}.{parent_name} FOR VALUES FROM ('{_from}-01-01') TO ('{to}-01-01');\n"
+        stmt = f"DROP TABLE {schema}.{parent_name}_p{_from}_{to};\n"
+        f.write(stmt)
+
+    def create_partition_table(
+            self, f, schema: str, parent_name: str, _from: str, to: str
+    ):
+        stmt = f"CREATE TABLE {schema}.{parent_name}_p{_from}_{to} PARTITION OF {schema}.{parent_name} FOR VALUES FROM ('{_from}-01-01') TO ('{to}-01-01');\n"
         f.write(stmt)
 
     def add_partitioning_constraint(
-        self, tbl: Table, column: str = "effective_start_dt"
+            self, tbl: Table, column: str = "effective_start_dt"
     ):
         tbl.dialect_options["postgresql"]["partition_by"] = f"RANGE({column})"
 
 
 class RedshiftProvider(MigrationProvider):
+
+    def create_table(self, name: str, schema: str, columns: typing.List[Column], primary_key_columns: typing.List[str],
+                     **kwargs) -> Table:
+        dist_key_columns = ['knowledge_hash']
+
+        dist_columns = list(filter(lambda x: 'identity' in x.name, columns))
+        if len(dist_columns) > 0:
+            dist_key_columns = dist_columns
+
+        return super(RedshiftProvider, self).create_table(name, schema, columns, primary_key_columns,
+                                                          redshift_distkey=dist_key_columns[0],
+                                                          redshift_sortkey=['effective_start_dt', 'effective_end_dt',
+                                                                            'knowledge_start_dt', 'knowledge_end_dt',
+                                                                            'valid_start_dt', 'valid_end_dt']
+                                                          )
+
     def supports_indices(self):
         return False
 
@@ -139,24 +178,24 @@ class MigrationProviderFactory(object):
 
 
 def create_schema_if_needed(
-    f, provider: MigrationProvider, doc: typing.Dict[str, typing.Any]
+        f_up, f_down, provider: MigrationProvider, doc: typing.Dict[str, typing.Any]
 ):
     required_schemas = [doc["provider"]]
 
     for r in doc["resources"]:
         required_schemas.append(r["dataset"])
 
-    f.write(f"CREATE SCHEMA IF NOT EXISTS temp_data;\n")
-
     for rs in set(required_schemas):
-        f.write(f"CREATE SCHEMA IF NOT EXISTS {rs};\n")
+        f_up.write(f"CREATE SCHEMA {rs};\n")
+        f_down.write(f"DROP SCHEMA {rs};\n")
 
 
 def create_table_if_needed(
-    f,
-    provider: MigrationProvider,
-    r: typing.Dict[str, typing.Any],
-    schema: str,
+        f_up,
+        f_down,
+        provider: MigrationProvider,
+        r: typing.Dict[str, typing.Any],
+        schema: str,
 ):
     columns = []
     indices = [
@@ -193,7 +232,7 @@ def create_table_if_needed(
                     )
                 )
 
-    # add atttributes
+    # add attributes
     for c in r["schema"]["attributes"]:
         columns.append(
             Column(
@@ -213,31 +252,32 @@ def create_table_if_needed(
 
     columns.extend(provider.get_auxiliary_columns())
 
-    tbl = Table(
-        r["code"],
-        provider.get_metadata(),
-        *columns,
-        PrimaryKeyConstraint(
-            "knowledge_hash",
-            "payload_hash",
-            "knowledge_end_dt",
-            "effective_start_dt",
-            "valid_start_dt",
-        ),
-        schema=schema,
+    tbl = provider.create_table(
+        r['code'],
+        schema,
+        columns,
+        ['knowledge_hash', 'payload_hash', 'knowledge_end_dt', 'effective_start_dt', 'valid_start_dt']
     )
 
     if provider.supports_partitioning():
         provider.add_partitioning_constraint(tbl)
 
-    tbl_s = (
+    tbl_up_statement = (
         str(
             CreateTable(tbl).compile(dialect=provider.get_sqlalchemy_dialect())
         )
-        .replace("CREATE TABLE", "CREATE TABLE IF NOT EXISTS")
-        .rstrip()
+            .rstrip()
     )
-    f.write(f"{tbl_s};\n")
+
+    tbl_down_statement = (
+        str(
+            DropTable(tbl).compile(dialect=provider.get_sqlalchemy_dialect())
+        )
+            .rstrip()
+    )
+
+    f_up.write(f"{tbl_up_statement};\n")
+    f_down.write(f"{tbl_down_statement};\n")
 
     if provider.supports_indices():
         for i in indices:
@@ -250,14 +290,15 @@ def create_table_if_needed(
                 CreateIndex(ix).compile(
                     dialect=provider.get_sqlalchemy_dialect()
                 )
-            ).replace("CREATE INDEX", "CREATE INDEX IF NOT EXISTS")
-            f.write(f"{cx};\n")
+            )
+
+            f_up.write(f"{cx};\n")
 
     if provider.supports_partitioning():
         ranges = provider.get_partition_ranges()
         for i in range(0, len(ranges) - 1):
             provider.create_partition_table(
-                f, schema, r["code"], str(ranges[i]), str(ranges[i + 1])
+                f_up, schema, r["code"], str(ranges[i]), str(ranges[i + 1])
             )
 
     return tbl
@@ -298,96 +339,78 @@ def map_attribute_type(t: str, col):
     raise ValueError(f"Invalid field type `{t}`.")
 
 
-def handle_schemas(
-    provider: MigrationProvider,
-    r: typing.Dict[str, typing.Any],
-    doc: typing.Dict[str, typing.Any],
+def handle_adapter_migrations():
+    return None, None
+
+
+def handle_resource_migrations(
+        provider: MigrationProvider,
+        r: typing.Dict[str, typing.Any],
+        doc: typing.Dict[str, typing.Any],
 ):
-    dest_file = os.path.join(
-        "migrations", r["code"], provider.get_name(), UP_INITIAL_MIGRATION_NAME
+    up_file = os.path.join(
+        "migrations", r["code"], provider.get_name(), UP_TABLES_MIGRATION_NAME
     )
+
+    down_file = os.path.join(
+        "migrations", r["code"], provider.get_name(), DOWN_TABLES_MIGRATION_NAME
+    )
+
     pathlib.Path(
         os.path.join("migrations", r["code"], provider.get_name())
     ).mkdir(parents=True, exist_ok=True)
 
-    click.echo(f"Adding initial migrations to `{dest_file}` ...")
+    click.echo(f"Adding table migrations to `{up_file}` ...")
 
-    with open(dest_file, "w") as f:
-        create_schema_if_needed(f, provider, doc)
+    with open(up_file, "w") as f_up:
+        with open(down_file, 'w') as f_down:
+            create_table_if_needed(f_up, f_down, provider, r, doc["provider"])
 
-    click.echo(f"Initial migrations sucessfully written to `{dest_file}`.")
-    return dest_file
+    click.echo(f"Table up migrations successfully written to `{up_file}`.")
+    click.echo(f"Table down  migrations successfully written to `{down_file}`.")
 
-
-def handle_tables(
-    provider: MigrationProvider,
-    r: typing.Dict[str, typing.Any],
-    doc: typing.Dict[str, typing.Any],
-):
-    dest_file = os.path.join(
-        "migrations", r["code"], provider.get_name(), TABLES_MIGRATION_NAME
-    )
-    pathlib.Path(
-        os.path.join("migrations", r["code"], provider.get_name())
-    ).mkdir(parents=True, exist_ok=True)
-
-    click.echo(f"Adding table migrations to `{dest_file}` ...")
-    with open(dest_file, "w") as f:
-        create_table_if_needed(f, provider, r, doc["provider"])
-    click.echo(f"Table migrations sucessfully written to `{dest_file}`.")
-
-    return dest_file
+    return up_file, down_file
 
 
 def generate_initial_migration_for_resource(
-    provider: MigrationProvider,
-    r: typing.Dict[str, typing.Any],
-    doc: typing.Dict[str, typing.Any],
-    bucket: str,
-    adapter: str,
+        provider: MigrationProvider,
+        r: typing.Dict[str, typing.Any],
+        doc: typing.Dict[str, typing.Any],
+        bucket: str,
+        upload: bool
 ):
     client = boto3.client("s3")
 
-    schema_migration_file = handle_schemas(provider, r, doc)
-    tbl_migration_file = handle_tables(provider, r, doc)
-
-    schema_migration_dst_path = (
-        adapter
-        + "/"
-        + r["code"]
-        + "/"
-        + provider.get_name()
-        + "/"
-        + os.path.basename(schema_migration_file)
-    )
-    click.echo(f"Uploading schema migration to `{schema_migration_dst_path}.")
-    client.upload_file(
-        schema_migration_file, bucket, schema_migration_dst_path
-    )
+    tbl_migration_file_up, tbl_migration_file_down = handle_resource_migrations(provider, r, doc)
 
     tbl_migration_dst_path = (
-        adapter
-        + "/"
-        + r["code"]
-        + "/"
-        + provider.get_name()
-        + "/"
-        + os.path.basename(tbl_migration_file)
-    )
-    click.echo(f"Uploading table migration to `{tbl_migration_dst_path}.")
+            doc['code']
+            + "/"
+            + r["code"]
+            + "/"
+            + provider.get_name()
+            + "/"
 
-    client.upload_file(tbl_migration_file, bucket, tbl_migration_dst_path)
+    )
+
+    if upload:
+        click.echo(f"Uploading table migration to `{tbl_migration_dst_path}.")
+
+        client.upload_file(tbl_migration_file_up, bucket,
+                           tbl_migration_dst_path + os.path.basename(tbl_migration_file_up))
+        client.upload_file(tbl_migration_file_down, bucket,
+                           tbl_migration_dst_path + os.path.basename(tbl_migration_file_down))
 
 
 def generate_initial_migrations(
-    provider: MigrationProvider,
-    doc: typing.Dict[str, typing.Any],
-    bucket: str,
-    adapter: str,
+        provider: MigrationProvider,
+        doc: typing.Dict[str, typing.Any],
+        bucket: str,
+        upload: bool
 ):
     for r in doc["resources"]:
         generate_initial_migration_for_resource(
-            provider, r, doc, bucket, adapter
+            provider, r, doc, bucket, upload
         )
 
 
@@ -395,7 +418,6 @@ def generate_initial_migrations(
 @click.option(
     "--metadata", "-m", required=True, help="The metadata resource path."
 )
-@click.option("--adapter", "-a", required=True, help="The adapter name")
 @click.option(
     "--warehouse",
     "-w",
@@ -410,12 +432,19 @@ def generate_initial_migrations(
     required=True,
     help="The bucket containing the migrations.",
 )
-def main(metadata, adapter, warehouse, bucket):
+@click.option(
+    "--upload",
+    "-u",
+    default=True,
+    required=True,
+    help="The bucket containing the migrations.",
+)
+def main(metadata, warehouse, bucket, upload):
     with open(metadata, "r") as f:
         doc = json.load(f)
         for w in warehouse:
             generate_initial_migrations(
-                MigrationProviderFactory.get(w), doc, bucket, adapter
+                MigrationProviderFactory.get(w), doc, bucket, upload
             )
 
 
